"""
@file
    obsw_config.py
@date
    01.11.2019
@brief
    Global settings for UDP client
"""

import struct
import pprint
import logging
from config.obsw_definitions import ModeList, ComInterfaces

"""
Mission/Device specific information.
"""

# Object IDs

# Commands

"""
Other global variables
"""

# TMTC Client
G_TMTC_LOGGER_NAME = "TMTC Logger"
G_ERROR_LOG_FILE_NAME = "tmtc_error.log"
G_PP = pprint.PrettyPrinter()
LOGGER = logging.getLogger(G_TMTC_LOGGER_NAME)
# General Settings
G_SCRIPT_MODE = 1
G_MODE_ID: ModeList = ModeList.ListenerMode
G_SERVICE = 17
G_OP_CODE = 0
G_LISTENER_AFTER_OP = False
G_DISPLAY_MODE = "long"

# General TMTC Settings
G_APID = 0x65  # Global APID for EIVE

# Print Settings
G_PRINT_TO_FILE = True
G_PRINT_HK_DATA = False
G_PRINT_RAW_TM = False
G_PRINT_TM = True
G_RESEND_TC = False

# Time related
G_TM_TIMEOUT = 6
G_TC_SEND_TIMEOUT_FACTOR = 2.0

# Serial communication
# Binary Upload Settings
G_MAX_BINARY_FRAME_LENGTH = 1500
G_MAX_APP_DATA_LENGTH = G_MAX_BINARY_FRAME_LENGTH - 100

G_COM_IF: ComInterfaces = ComInterfaces.QEMU
# COM Port for serial communication
G_COM_PORT = 'COM0'
G_SERIAL_TIMEOUT = 0.01
G_SERIAL_BAUDRATE = 230400
G_SERIAL_FRAME_SIZE = 256
G_SERIAL_DLE_MAX_QUEUE_LEN = 25
G_SERIAL_DLE_MAX_FRAME_SIZE = 1024

# Ethernet communication
# Ethernet connection settings
# Bind to localhost, change accordingly!
G_ETHERNET_SEND_ADDRESS_IP = '127.0.0.1'
G_ETHERNET_SEND_PORT = 7301
# Bind to all interfaces, insecure!
G_ETHERNET_RECV_ADDRESS_IP = ''
G_ETHERNET_RECV_PORT = 7302

G_ETHERNET_RECV_ADDRESS = (G_ETHERNET_RECV_ADDRESS_IP, G_ETHERNET_RECV_PORT)
G_ETHERNET_SEND_ADDRESS = (G_ETHERNET_SEND_ADDRESS_IP, G_ETHERNET_SEND_PORT)

G_PP = pprint.PrettyPrinter()
LOGGER = logging.getLogger(G_TMTC_LOGGER_NAME)
# General Settings
G_SCRIPT_MODE = 1
G_MODE_ID = 0
G_SERVICE = 17
G_DISPLAY_MODE = "long"
G_COM_IF = 2
# COM Port for serial communication
G_COM_PORT = 'COM0'
"""
These objects are set for the Unit Test, no better solution found yet
"""
G_TM_LISTENER = None
G_COM_INTERFACE = None
G_TMTC_PRINTER = None

# noinspection PyUnusedLocal
def set_globals(args):
    global G_ETHERNET_RECV_ADDRESS, G_ETHERNET_SEND_ADDRESS, G_SCRIPT_MODE, G_MODE_ID, G_SERVICE, \
        G_DISPLAY_MODE, G_COM_IF, G_COM_PORT, G_SERIAL_TIMEOUT, G_TM_TIMEOUT, \
        G_TC_SEND_TIMEOUT_FACTOR, G_PRINT_TO_FILE, G_PRINT_HK_DATA, G_PRINT_RAW_TM, G_PRINT_TM, \
        G_OP_CODE, G_RESEND_TC, G_LISTENER_AFTER_OP

    if args.shortDisplayMode:
        G_DISPLAY_MODE = "short"
    else:
        G_DISPLAY_MODE = "long"
    if 0 <= args.mode <= 6:
        if args.mode == 0:
            G_MODE_ID = ModeList.GUIMode
        elif args.mode == 1:
            G_MODE_ID = ModeList.ListenerMode
        elif args.mode == 2:
            G_MODE_ID = ModeList.SingleCommandMode
        elif args.mode == 3:
            G_MODE_ID = ModeList.ServiceTestMode
        elif args.mode == 4:
            G_MODE_ID = ModeList.SoftwareTestMode
        elif args.mode == 5:
            G_MODE_ID = ModeList.BinaryUploadMode
        elif args.mode == 6:
            G_MODE_ID = ModeList.UnitTest
    else:
        G_MODE_ID = ModeList[1]

    if args.com_if == ComInterfaces.Ethernet.value:
        G_COM_IF = ComInterfaces.Ethernet
    elif args.com_if == ComInterfaces.Serial.value:
        G_COM_IF = ComInterfaces.Serial
    elif args.com_if == ComInterfaces.QEMU.value:
        G_COM_IF = ComInterfaces.QEMU
    else:
        G_COM_IF = ComInterfaces.Dummy

    G_SERVICE = str(args.service)
    if G_SERVICE.isdigit():
        G_SERVICE = int(args.service)
    else:
        G_SERVICE = args.service

    G_OP_CODE = str(args.op_code)
    if G_OP_CODE.isdigit():
        G_OP_CODE = int(G_OP_CODE)
    else:
        G_OP_CODE = str(G_OP_CODE)

    G_MODE_ID = G_MODE_ID
    G_PRINT_HK_DATA = args.print_hk
    G_PRINT_TM = args.print_tm
    G_PRINT_TO_FILE = args.print_log
    G_PRINT_RAW_TM = args.rawDataPrint
    G_COM_PORT = args.com_port
    G_TM_TIMEOUT = args.tm_timeout
    G_RESEND_TC = args.resend_tc
    G_LISTENER_AFTER_OP = args.listener

    from config.obsw_user_code import global_setup_hook
    global_setup_hook()

